--------------------------------------------------------------------------------
-- [VHDL Praktikum SS2004]
--
-- Aufgabe 3:     Minima-Rechenwerk
-- Entity:        E_MINIMA (Top-Level-Entity)               
--
-- Andreas Schibilla (ii4900)
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;


--------------------------------------------------------------------------------
-- Impuls-Generation-Entity    
--------------------------------------------------------------------------------
entity E_IMPULSE is    
    port (
        CLK:         in  std_logic;
        RST:         in  std_logic; 
        A_IMPULS:    out std_logic;
        B_IMPULS:    out std_logic;
        C_IMPULS:    out std_logic;
        CNT_OUT:     out STD_LOGIC_VECTOR (4 DOWNTO 0)
    );
end E_IMPULSE;


--------------------------------------------------------------------------------
-- Impuls-Generation-Architecture
-- Erzeugt die drei Steuerimpulse A,B,C aus dem Takt mit Hilfe eines Zhlers
--------------------------------------------------------------------------------
architecture A_IMPULSE of E_IMPULSE is

component COUNTER
   PORT
   (
      clock    : IN STD_LOGIC ;
      aclr     : IN STD_LOGIC ;
      q        : OUT STD_LOGIC_VECTOR (4 DOWNTO 0)
   );
end component;

signal  INT_CLEAR    : std_logic;
signal  INT_COUNTER  : STD_LOGIC_VECTOR (4 DOWNTO 0);
signal  INT_B_IMPULS : std_logic;

begin      
 -- Zhler via Port Map einbinden
 COUNTER_inst : COUNTER PORT MAP (  clock    => CLK,
                                    aclr     => RST,
                                    q        => INT_COUNTER
                                 );                                  
 
 -- Counter nach auen geben                                
 CNT_OUT    <= INT_COUNTER;
 -- B_Impuls zuweisen, ist Zhler Null noch nicht auf high gehen
 B_IMPULS   <= INT_B_IMPULS and (INT_COUNTER(0) or INT_COUNTER(1) or INT_COUNTER(2) or INT_COUNTER(3) or INT_COUNTER(4));
 
 -- Prozess um die Impulse zu erzeugen
 P2: process(CLK, RST)
 begin
   if RST = '1' then
      A_IMPULS       <= '0';
      INT_B_IMPULS   <= '0';
      C_IMPULS       <= '0';
      
   elsif rising_edge(CLK) then
      if INT_COUNTER = "00000" then                   -- mit erstem Counter-Stand B-Impuls setzen
         INT_B_IMPULS <= '1';
      else 
         INT_B_IMPULS <= '0';
      end if;

      if INT_COUNTER = "00100" then                   -- fnfter Takt erzeugt A-Impuls
         A_IMPULS <= '1';
      elsif INT_COUNTER = "11000" then
         A_IMPULS <= '0';
      end if;

      if INT_COUNTER = "01100" then                   -- 8 Takte spter wird C-Impuls angesetzt
         C_IMPULS <= '1';
      elsif INT_COUNTER = "11000" then
         C_IMPULS <= '0';
      end if;
      
      
   end if;
 end process P2;
              
end A_IMPULSE;




